%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PHY306, Assignment 5, Numerics question template.  IISER Bhopal  2020
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function Assignment5

Ncomponents = 3;   %%%%%%% POSITION 1
Npts = 64;
L = 1;
rho0=1;
xi = 0.1;

xaxis = linspace(-L,L,Npts);
yaxis = linspace(-L,L,Npts);
zaxis = linspace(-L,L,Npts);
[X Y Z] = ndgrid(xaxis,yaxis,zaxis);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Assemble rigid body pieces (currently snow-statue)
%%%%%%% POSITION 2
% (spheres)
centre = [0,0,0.6];
radius = 0.2;
R = sqrt((X-centre(1)).^2 + (Y-centre(2)).^2 + (Z-centre(3)).^2);
rigid_body_components(1,:,:,:)=rho0*0.5*(tanh(-(R-radius)/xi)+1);

centre = [0,0,0.1];
radius = 0.3;
R = sqrt((X-centre(1)).^2 + (Y-centre(2)).^2 + (Z-centre(3)).^2);
rigid_body_components(2,:,:,:)=rho0*0.5*(tanh(-(R-radius)/xi)+1);

centre = [0,0,-0.6];
radius = 0.4;
R = sqrt((X-centre(1)).^2 + (Y-centre(2)).^2 + (Z-centre(3)).^2);
rigid_body_components(3,:,:,:)=rho0*0.5*(tanh(-(R-radius)/xi)+1);


% (cylinders)
%along x direction
length = 0.2;
centre = [0.2+length/2,0,0.55];
radius = 0.01;

R_cyl = sqrt((Y-centre(2)).^2 + (Z-centre(3)).^2);
rigid_body_components(4,:,:,:)=rho0*0.5*(tanh((X-centre(1)+length/2)/xi) + tanh(-(X-centre(1)-length/2)/xi)).*(tanh(-(R_cyl-radius)/xi)+1);

%along y direction
length = 1.5;
centre = [0,0,0.1];
radius = 0.03;

R_cyl = sqrt((X-centre(2)).^2 + (Z-centre(3)).^2);
rigid_body_components(5,:,:,:)=rho0*0.5*(tanh((Y-centre(1)+length/2)/xi) + tanh(-(Y-centre(1)-length/2)/xi)).*(tanh(-(R_cyl-radius)/xi)+1);

% (cuboids)
centre = [0,0,0];
widths = [0.3,0.2,0.45];
%rigid_body_components(1,:,:,:)=rho0*(tanh((X-centre(1)+widths(1))/xi) + tanh(-(X-centre(1)-widths(1))/xi)).*(tanh((Y-centre(2)+widths(2))/xi) + tanh(-(Y-centre(2)-widths(2))/xi)).*(tanh((Z-centre(3)+widths(3))/xi) + tanh(-(Z-centre(3)-widths(3))/xi))/8;



rigid_body = squeeze(sum(rigid_body_components(1:Ncomponents,:,:,:),1));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3D visualisation of density distribution
rhocut = rho0/2.0;

p=patch(isosurface(xaxis,yaxis,zaxis,rigid_body,rhocut));
isonormals(xaxis,yaxis,zaxis,rigid_body,p)
%axis tight
DiffuseStrength = 0.4;
SpecularStrength = 0.8;
Alpha = 1; %0=transparent, 1=opaque
render_opengl=1;

set(p,'FaceColor','green','EdgeColor','none','DiffuseStrength',DiffuseStrength,'SpecularStrength',SpecularStrength,'FaceAlpha',Alpha);
daspect([1 1 1])
viewvec = [0.2 1 0.2];
view(viewvec)
CameraUpVector = ([0 0 1]);
camup(CameraUpVector)

%lighting_bugfix phong
delete(findall(gcf,'Type','light'))
lighting phong
camlight right

hdllight = light;
set(hdllight,'Position',viewvec-[0 1 -1])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Numerical "integration" to get inertia tensor
%   see also "doc trapz"
dx = xaxis(2)-xaxis(1);
dy = yaxis(2)-yaxis(1);
dz = zaxis(2)-zaxis(1);
dV=dx*dy*dz;

%%%%%%% POSITION 3
Ixx = sum(sum(sum(rigid_body.*(Y.*Y + Z.*Z))))*dV;
Iyy = XXX;
Izz = XXX;
Ixy = XXX;
Ixz = XXX;
Iyz = XXX;

Itensor= [Ixx Ixy Ixz
		  Ixy Iyy Iyz
		  Ixz Iyz Izz];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Diagonalisation of inertia tensor

[VV II] = eig(Itensor); % see "doc eig"
% sort eigenvalues in descending order and correspondingly reorder eigenvectors
[II indices] = sort(diag(II),'descend');
VV = VV(:,indices);

display(['Found principal moments of inertia: ',num2str(II(1)),' (red), ',num2str(II(2)),' (black), ',num2str(II(3)),' (blue)'])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Visualisation of principal axes
hold on
arrowproperties1=[0.3 0.6];
arrowproperties2=[40 40];

gap = 1.2;
length = 0.3;

arrow3d(gap*transpose(VV(:,1)),(gap+length)*transpose(VV(:,1)),40,'cylinder',arrowproperties1,arrowproperties2,'red')
arrow3d(gap*transpose(VV(:,2)),(gap+length)*transpose(VV(:,2)),40,'cylinder',arrowproperties1,arrowproperties2,'black')
arrow3d(gap*transpose(VV(:,3)),(gap+length)*transpose(VV(:,3)),40,'cylinder',arrowproperties1,arrowproperties2,'blue')
hold off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Final potting

%xlim([-L L])
%ylim([-L L])
%zlim([-L L])
xlabel(['x'])
ylabel(['y'])
zlabel(['z'])

if(render_opengl),
  set(gcf,'Renderer','opengl')
else  
  set(gcf,'Renderer',' ')
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lw=2;
fs=20;

lineobj = findobj('type', 'Line');
set(lineobj,'LineWidth', lw);

patchobj = findobj('type', 'Patch');
set(patchobj,'LineWidth', lw);

textobj = findobj('type', 'axes');
set(textobj,'LineWidth', lw);
set(textobj, 'fontsize', fs);
set(textobj, 'FontName', 'Times');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%save(['assignment5_output.mat'])
%display(['Wrote assignment5_output.mat'])


%end % main function